<?php

declare(strict_types=1);

namespace Drupal\jsonapi_address;

use CommerceGuys\Addressing\Country\Country;
use CommerceGuys\Addressing\Country\CountryRepositoryInterface;
use CommerceGuys\Addressing\Subdivision\SubdivisionRepositoryInterface;
use Drupal\Core\ParamConverter\ParamConverterInterface;
use Drupal\Core\ParamConverter\ParamNotConvertedException;
use Symfony\Component\Routing\Route;

/**
 * Custom parameter converter for countries and subdivisions.
 */
final class AddressParamConverter implements ParamConverterInterface {

  public function __construct(
    private readonly CountryRepositoryInterface $countryRepository,
    private readonly SubdivisionRepositoryInterface $subdivisionRepository,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function convert($value, $definition, $name, array $defaults) {
    $type_part = strstr($definition['type'], ':');
    if (!$type_part) {
      throw new ParamNotConvertedException(sprintf('The type definition "%s" is invalid. The expected format is "jsonapi_address:<type>".', $definition['type']));
    }
    $type = substr($type_part, 1);
    switch ($type) {
      case 'country':
        // Country code *should* only be two uppercase characters.
        if (!preg_match('/^[A-Z]{2}$/', $value)) {
          throw new ParamNotConvertedException(sprintf('"%s" is not a valid ISO-3166-1 alpha-2 country code.', $value));
        }
        try {
          return $this->countryRepository->get($value);
        }
        catch (\Exception $exception) {
          throw new ParamNotConvertedException('A country must be a two-letter ISO 3166-1 alpha-2 country code.', $exception);
        }
      case 'subdivision':
        // We expect the country to have been up-cast just before us.
        if (empty($defaults['country']) || !$defaults['country'] instanceof Country) {
          throw new ParamNotConvertedException('A subdivision must be qualified by a country.');
        }

        return $this->subdivisionRepository->get($value, [$defaults['country']]);

      default:
        return NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function applies($definition, $name, Route $route): bool {
    return isset($definition['type']) && str_starts_with($definition['type'], 'jsonapi_address:');
  }

}
