<?php

declare(strict_types=1);

namespace Drupal\jsonapi_address;

use CommerceGuys\Addressing\Country\Country;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Url;
use Drupal\jsonapi\JsonApiResource\Link;
use Drupal\jsonapi\JsonApiResource\LinkCollection;
use Drupal\jsonapi\JsonApiResource\ResourceObject;
use Drupal\jsonapi\ResourceType\ResourceType;
use Drupal\jsonapi\ResourceType\ResourceTypeAttribute;

/**
 * JSON:API Resource builder for countries.
 */
final class CountryResourceBuilder {

  /**
   * Builds a country resource.
   *
   * @param \CommerceGuys\Addressing\Country\Country $country
   *   The country.
   *
   * @return \Drupal\jsonapi\JsonApiResource\ResourceObject
   *   Returns a resource object.
   */
  public function build(Country $country): ResourceObject {
    $cache = new CacheableMetadata();
    $cache->setCacheMaxAge(Cache::PERMANENT);
    $links = new LinkCollection([
      'self' => new Link($cache, new Url('jsonapi_address.country_single', [
        'country' => $country->getCountryCode(),
      ]), 'self'),
      'subdivisions' => new Link($cache, new Url('jsonapi_address.subdivision_collection', [
        'country' => $country->getCountryCode(),
      ]), 'subdivisions'),
    ]);
    return new ResourceObject(
      $cache,
      $this->getResourceType(),
      $country->getCountryCode(),
      NULL,
      [
        'country_code' => $country->getCountryCode(),
        'name' => $country->getName(),
        'three_letter_code' => $country->getThreeLetterCode(),
        'numeric_code' => $country->getNumericCode(),
        'currency_code' => $country->getCurrencyCode(),
        'locale' => $country->getLocale(),
      ],
      $links
    );
  }

  /**
   * Returns the resource type definition for the given context.
   *
   * @return \Drupal\jsonapi\ResourceType\ResourceType
   *   The resource type containing field definitions and configurations.
   */
  public function getResourceType() : ResourceType {
    $fields = [
      'country_code' => new ResourceTypeAttribute('country_code', 'country_code', TRUE, TRUE),
      'name' => new ResourceTypeAttribute('name', 'name', TRUE, TRUE),
      'three_letter_code' => new ResourceTypeAttribute('three_letter_code', 'three_letter_code', TRUE, TRUE),
      'numeric_code' => new ResourceTypeAttribute('numeric_code', 'numeric_code', TRUE, TRUE),
      'currency_code' => new ResourceTypeAttribute('currency_code', 'currency_code', TRUE, TRUE),
      'locale' => new ResourceTypeAttribute('locale', 'locale', TRUE, TRUE),
    ];

    $resource_type = new ResourceType('address', 'country', NULL, FALSE, TRUE, FALSE, FALSE, $fields);
    $resource_type->setRelatableResourceTypes([]);
    return $resource_type;
  }

}
