<?php

declare(strict_types=1);

namespace Drupal\jsonapi_address;

use CommerceGuys\Addressing\AddressFormat\AddressFormatRepositoryInterface;
use CommerceGuys\Addressing\Country\Country;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Url;
use Drupal\jsonapi\JsonApiResource\Link;
use Drupal\jsonapi\JsonApiResource\LinkCollection;
use Drupal\jsonapi\JsonApiResource\ResourceObject;
use Drupal\jsonapi\ResourceType\ResourceType;
use Drupal\jsonapi\ResourceType\ResourceTypeAttribute;

/**
 * JSON:API Resource builder for formatting.
 */
final class FormatResourceBuilder {

  public function __construct(protected readonly AddressFormatRepositoryInterface $formatRepository) {}

  /**
   * Builds a country resource.
   *
   * @param \CommerceGuys\Addressing\Country\Country $country
   *   The country.
   *
   * @return \Drupal\jsonapi\JsonApiResource\ResourceObject
   *   Returns a resource object.
   */
  public function build(Country $country): ResourceObject {
    $format = $this->formatRepository->get($country->getCountryCode());
    $cache = new CacheableMetadata();
    $cache->setCacheMaxAge(Cache::PERMANENT);
    $links = new LinkCollection([
      'self' => new Link($cache, new Url('jsonapi_address.format', [
        'country' => $country->getCountryCode(),
      ]), 'self'),
    ]);
    return new ResourceObject(
      $cache,
      $this->getResourceType(),
      $format->getCountryCode(),
      NULL,
      [
        'country_code' => $format->getCountryCode(),
        'locale' => $format->getLocale(),
        'format' => $format->getFormat(),
        'local_format' => $format->getLocalFormat(),
        'used_fields' => $format->getUsedFields(),
        'required_fields' => $format->getRequiredFields(),
        'uppercase_fields' => $format->getUppercaseFields(),
        'default_values' => $format->getDefaultValues(),
        'administrative_area_type' => $format->getAdministrativeAreaType(),
        'locality_type' => $format->getLocalityType(),
        'dependent_locality_type' => $format->getDependentLocalityType(),
        'postal_code_type' => $format->getPostalCodeType(),
        'postal_code_pattern' => $format->getPostalCodePattern(),
        'postal_code_prefix' => $format->getPostalCodePrefix(),
        'subdivision_depth' => $format->getSubdivisionDepth(),
      ],
      $links
    );
  }

  /**
   *
   * @return \Drupal\jsonapi\ResourceType\ResourceType
   */
  public function getResourceType() : ResourceType {
    $fields = [
      'country_code' => new ResourceTypeAttribute('country_code', 'country_code', TRUE, TRUE),
      'locale' => new ResourceTypeAttribute('locale', 'locale', TRUE, TRUE),
      'format' => new ResourceTypeAttribute('format', 'format', TRUE, TRUE),
      'local_format' => new ResourceTypeAttribute('local_format', 'local_format', TRUE, TRUE),
      'used_fields' => new ResourceTypeAttribute('used_fields', 'used_fields', TRUE, FALSE),
      'required_fields' => new ResourceTypeAttribute('required_fields', 'required_fields', TRUE, FALSE),
      'uppercase_fields' => new ResourceTypeAttribute('uppercase_fields', 'uppercase_fields', TRUE, FALSE),
      'default_values' => new ResourceTypeAttribute('default_values', 'default_values', TRUE, FALSE),
      'administrative_area_type' => new ResourceTypeAttribute('administrative_area_type', 'administrative_area_type', TRUE, TRUE),
      'locality_type' => new ResourceTypeAttribute('locality_type', 'locality_type', TRUE, TRUE),
      'dependent_locality_type' => new ResourceTypeAttribute('dependent_locality_type', 'dependent_locality_type', TRUE, TRUE),
      'postal_code_type' => new ResourceTypeAttribute('postal_code_type', 'postal_code_type', TRUE, TRUE),
      'postal_code_pattern' => new ResourceTypeAttribute('postal_code_pattern', 'postal_code_pattern', TRUE, TRUE),
      'postal_code_prefix' => new ResourceTypeAttribute('postal_code_prefix', 'postal_code_prefix', TRUE, TRUE),
      'subdivision_depth' => new ResourceTypeAttribute('subdivision_depth', 'subdivision_depth', TRUE, TRUE),
    ];

    $resource_type = new ResourceType('address', 'format', NULL, FALSE, TRUE, FALSE, FALSE, $fields);
    $resource_type->setRelatableResourceTypes([]);
    return $resource_type;
  }

}
