<?php

declare(strict_types=1);

namespace Drupal\jsonapi_address;

use CommerceGuys\Addressing\Country\Country;
use CommerceGuys\Addressing\Subdivision\Subdivision;
use Drupal\address\Repository\AddressFormatRepository;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\jsonapi\JsonApiResource\LinkCollection;
use Drupal\jsonapi\JsonApiResource\ResourceObject;
use Drupal\jsonapi\ResourceType\ResourceType;
use Drupal\jsonapi\ResourceType\ResourceTypeAttribute;

/**
 * Class to handle postal code validation resources.
 */
final class PostalCodeValidationResourceBuilder {

  /**
   * Constructs a PostalCodeValidationResourceBuilder object.
   */
  public function __construct(private readonly AddressFormatRepository $formatRepository) {
  }

  /**
   * Builds the validation resource.
   */
  public function build(Country $country, string $postal_code, ?Subdivision $subdivision = NULL): ResourceObject {
    $format = $this->formatRepository->get($country->getCountryCode());
    if ($subdivision && $subdivision->getPostalCodePattern()) {
      $format = $subdivision->getPostalCodePattern();
    }

    $pattern = $format->getPostalCodePattern();
    // Safe delimiter (#) + ^…$ anchors + case-insensitive flag.
    $regex = '#^' . $pattern . '$#i';
    $is_valid = preg_match($regex, $postal_code) === 1;

    $cache = new CacheableMetadata();
    $cache->setCacheMaxAge(0);
    $links = new LinkCollection([]);
    return new ResourceObject(
      $cache,
      $this->getResourceType(),
      $country->getCountryCode() . '-' . $postal_code,
      NULL,
      [
        'country_code' => $country->getCountryCode(),
        'postal_code' => $postal_code,
        'is_valid' => $is_valid,
      ],
      $links,
    );
  }

  /**
   * Retrieves the resource type definition for postal code validation.
   *
   * @return \Drupal\jsonapi\ResourceType\ResourceType
   *   The resource type object.
   */
  public function getResourceType(): ResourceType {
    $fields = [
      'country_code' => new ResourceTypeAttribute('country_code', 'country_code', TRUE, TRUE),
      'postal_code' => new ResourceTypeAttribute('postal_code', 'postal_code', TRUE, TRUE),
      'is_valid' => new ResourceTypeAttribute('is_valid', 'is_valid', TRUE, TRUE),
    ];

    return new ResourceType(
      'address',
      'postal_code_validation',
      NULL,
      FALSE,
      TRUE,
      FALSE,
      FALSE,
      $fields,
    );
  }

}
