<?php

declare(strict_types=1);

namespace Drupal\jsonapi_address\Resource;

use CommerceGuys\Addressing\Country\Country;
use CommerceGuys\Addressing\Subdivision\SubdivisionRepositoryInterface;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Http\Exception\CacheableBadRequestHttpException;
use Drupal\jsonapi\JsonApiResource\ResourceObjectData;
use Drupal\jsonapi\ResourceResponse;
use Drupal\jsonapi_address\PostalCodeValidationResourceBuilder;
use Drupal\jsonapi_resources\Resource\ResourceBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Route;

/**
 * Validates postal codes for a given country / subdivision.
 */
class PostalCodeValidation extends ResourceBase implements ContainerInjectionInterface {

  public function __construct(
    protected readonly SubdivisionRepositoryInterface $subdivisionRepository,
    protected readonly PostalCodeValidationResourceBuilder $resourceBuilder,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('address.subdivision_repository'),
      $container->get('jsonapi_address.postal_code_validation_resource_builder'),
    );
  }

  /**
   * Processes a request to validate a postal code for a given country / sub.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   * @param \CommerceGuys\Addressing\Country\Country $country
   *   The country object.
   *
   * @return \Drupal\jsonapi\ResourceResponse
   *   The JSON:API Response object.
   */
  public function process(Request $request, Country $country) : ResourceResponse {
    $postal_code = $request->query->get('postal_code');

    if (!$postal_code) {
      throw new CacheableBadRequestHttpException(new CacheableMetadata(), 'Missing postal code.');
    }

    $subdivision_code = $request->query->get('subdivision_code');
    // Validate the postal code if available and pass it to the builder.
    $subdivision = NULL;
    if ($subdivision_code) {
      $subdivision = $this->subdivisionRepository->get($subdivision_code, [$country]);
    }

    $resource = $this->resourceBuilder->build($country, $postal_code, $subdivision);
    $top_level_data = new ResourceObjectData([$resource], 1);
    return $this->createJsonapiResponse($top_level_data, $request);
  }

  /**
   * {@inheritdoc}
   */
  public function getRouteResourceTypes(Route $route, string $route_name) : array {

    return [$this->resourceBuilder->getResourceType()];
  }

}
