<?php

declare(strict_types=1);

namespace Drupal\jsonapi_address;

use CommerceGuys\Addressing\Subdivision\Subdivision;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Url;
use Drupal\jsonapi\JsonApiResource\Link;
use Drupal\jsonapi\JsonApiResource\LinkCollection;
use Drupal\jsonapi\JsonApiResource\ResourceObject;
use Drupal\jsonapi\ResourceType\ResourceType;
use Drupal\jsonapi\ResourceType\ResourceTypeAttribute;

/**
 * JSON:API Resource builder for subdivisions.
 */
final class SubdivisionResourceBuilder {

  /**
   * Builds a subdivision resource.
   *
   * @param \CommerceGuys\Addressing\Subdivision\Subdivision $subdivision
   *   The subdivision.
   *
   * @return \Drupal\jsonapi\JsonApiResource\ResourceObject
   *   Returns a resource object.
   */
  public function build(Subdivision $subdivision): ResourceObject {
    $cache = new CacheableMetadata();
    $cache->setCacheMaxAge(Cache::PERMANENT);
    $links = new LinkCollection([
      'self' => new Link($cache, new Url(
        'jsonapi_address.subdivision', [
          'country' => $subdivision->getCountryCode(),
          'subdivision' => $subdivision->getCode(),
        ]), 'self'),
    ]);
    return new ResourceObject(
      $cache,
      $this->getResourceType(),
      $subdivision->getCode(),
      NULL,
      [
        'parent' => $subdivision->getParent(),
        'country_code' => $subdivision->getCountryCode(),
        'code' => $subdivision->getCode(),
        'local_code' => $subdivision->getLocalCode(),
        'name' => $subdivision->getName(),
        'local_name' => $subdivision->getLocalName(),
        'postal_code_pattern' => $subdivision->getPostalCodePattern(),
        'children' => $subdivision->getChildren()->toArray(),
        'locale' => $subdivision->getLocale(),
      ],
      $links,
    );
  }

  /**
   * Retrieves the resource type for the subdivision resource.
   *
   * @return \Drupal\jsonapi\ResourceType\ResourceType
   *   The resource type definition.
   */
  public function getResourceType(): ResourceType {
    $fields = [
      'parent' => new ResourceTypeAttribute('parent', 'parent', TRUE, TRUE),
      'country_code' => new ResourceTypeAttribute('country_code', 'country_code', TRUE, TRUE),
      'local_code' => new ResourceTypeAttribute('local_code', 'local_code', TRUE, TRUE),
      'name' => new ResourceTypeAttribute('name', 'name', TRUE, TRUE),
      'local_name' => new ResourceTypeAttribute('local_name', 'local_name', TRUE, TRUE),
      'postal_code_pattern' => new ResourceTypeAttribute('postal_code_pattern', 'postal_code_pattern', TRUE, TRUE),
      // @todo Convert this to ResourceTypeRelationship
      'children' => new ResourceTypeAttribute('children', 'children', TRUE, FALSE),
      'locale' => new ResourceTypeAttribute('locale', 'locale', TRUE, TRUE),

    ];

    $resource_type = new ResourceType('address', 'subdivision', NULL, FALSE, TRUE, FALSE, FALSE, $fields);
    // @todo Add parents, children, and country.
    $resource_type->setRelatableResourceTypes([]);
    return $resource_type;
  }

}
